#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import unittest

import appier

from . import mock

class ModelTest(unittest.TestCase):

    def setUp(self):
        self.app = appier.App()
        self.app._register_models_m(mock, "Mocks")

    def tearDown(self):
        self.app.unload()
        adapter = appier.get_adapter()
        adapter.drop_db()

    def test_basic(self):
        person = mock.Person(fill = False)
        person.name = "Name"

        self.assertEqual(person.name, "Name")
        self.assertEqual(person["name"], "Name")
        self.assertEqual(len(person), 1)

        person["age"] = 20

        self.assertEqual(person.age, 20)
        self.assertEqual(person["age"], 20)
        self.assertEqual(len(person), 2)

        self.assertEqual("age" in person, True)
        self.assertEqual("boss" in person, False)
        self.assertEqual(bool(person), True)

        del person["name"]

        self.assertRaises(AttributeError, lambda: person.name)
        self.assertRaises(KeyError, lambda: person["name"])

        del person.age

        self.assertRaises(AttributeError, lambda: person.age)
        self.assertRaises(KeyError, lambda: person["age"])

        self.assertEqual(bool(person), False)

    def test_find(self):
        result = mock.Person.find(age = 1)
        self.assertEqual(len(result), 0)

        person = mock.Person()
        person.age = 1
        person.name = "Name"
        person.save()

        result = mock.Person.find(age = 1)
        self.assertEqual(len(result), 1)
        self.assertEqual(result[0].age, 1)

    def test_count(self):
        result = mock.Person.count()
        self.assertEqual(result, 0)

        person = mock.Person()
        person.age = 1
        person.name = "Name"
        person.save()

        result = mock.Person.count()
        self.assertEqual(result, 1)

    def test_delete(self):
        result = mock.Person.count()
        self.assertEqual(result, 0)

        person = mock.Person()
        person.age = 1
        person.name = "Name"
        person.save()

        result = mock.Person.count()
        self.assertEqual(result, 1)

        person.delete()

        result = mock.Person.count()
        self.assertEqual(result, 0)

    def test_validation(self):
        person = mock.Person()

        self.assertRaises(appier.ValidationError, person.save)

        person = mock.Person()
        person.name = "Name"
        person.save()

        person = mock.Person()
        person.name = "Name"

        self.assertRaises(appier.ValidationError, person.save)

    def test_map(self):
        person = mock.Person()
        person.name = "Name"

        self.assertEqual(person.name, "Name")

        person.save()

        self.assertEqual(person.identifier, 1)
        self.assertEqual(person.identifier_safe, 1)
        self.assertEqual(person.name, "Name")

        person_m = person.map()

        self.assertEqual(isinstance(person_m, dict), True)
        self.assertEqual(person_m["identifier"], 1)
        self.assertEqual(person_m["identifier_safe"], 1)
        self.assertEqual(person_m["name"], "Name")

        person.age = 20
        person.hidden = "Hidden"

        self.assertEqual(person.age, 20)
        self.assertEqual(person.hidden, "Hidden")

        person_m = person.map(all = True)

        self.assertEqual(isinstance(person_m, dict), True)
        self.assertEqual(person_m["identifier"], 1)
        self.assertEqual(person_m["identifier_safe"], 1)
        self.assertEqual(person_m["name"], "Name")
        self.assertEqual(person_m["age"], 20)
        self.assertEqual(person_m["hidden"], "Hidden")

        cat = mock.Cat()
        cat.name = "NameCat"

        self.assertEqual(cat.name, "NameCat")

        cat.save()

        self.assertEqual(cat.identifier, 1)

        person.cats = [cat]
        person.save()

        person_m = person.map(resolve = True, all = True)

        self.assertEqual(isinstance(person_m, dict), True)
        self.assertEqual(isinstance(person_m["cats"], list), True)
        self.assertEqual(isinstance(person_m["cats"][0], dict), True)
        self.assertEqual(person_m["cats"][0]["identifier"], 1)
        self.assertEqual(person_m["cats"][0]["identifier_safe"], 1)
        self.assertEqual(person_m["cats"][0]["name"], "NameCat")

        person = mock.Person.get(identifier = 1)

        self.assertEqual(person.cats[0].name, "NameCat")

        person = mock.Person.get(identifier = 1)

        person_m = person.map(all = True)

        self.assertEqual(person_m["cats"][0], 1)

        person_m = person.map(resolve = True, all = True)

        self.assertEqual(isinstance(person_m, dict), True)
        self.assertEqual(isinstance(person_m["cats"], list), True)
        self.assertEqual(isinstance(person_m["cats"][0], dict), True)
        self.assertEqual(person_m["cats"][0]["identifier"], 1)
        self.assertEqual(person_m["cats"][0]["identifier_safe"], 1)
        self.assertEqual(person_m["cats"][0]["name"], "NameCat")

    def test_increment(self):
        person = mock.Person()
        person.name = "Name1"
        person.save()

        self.assertEqual(person.identifier, 1)
        self.assertEqual(person.name, "Name1")

        person = mock.Person()
        person.name = "Name2"
        person.save()

        self.assertEqual(person.identifier, 2)
        self.assertEqual(person.name, "Name2")

        person = mock.Person()
        person.name = "Name3"
        person.save()

        self.assertEqual(person.identifier, 3)
        self.assertEqual(person.name, "Name3")

        person.delete()

        person = mock.Person()
        person.name = "Name4"
        person.save()

        self.assertEqual(person.identifier, 4)
        self.assertEqual(person.name, "Name4")

    def test_sort(self):
        person = mock.Person()
        person.name = "Name"
        person.save()

        other = mock.Person()
        other.name = "NameOther"
        other.save()

        result = mock.Person.find()

        self.assertEqual(result[0].identifier, person.identifier)
        self.assertEqual(result[1].identifier, other.identifier)

        result = mock.Person.find(sort = [("identifier", -1)])

        self.assertEqual(result[0].identifier, other.identifier)
        self.assertEqual(result[1].identifier, person.identifier)

        result = mock.Person.get(sort = [("identifier", -1)])

        self.assertEqual(result.identifier, other.identifier)

    def test_range(self):
        for index in range(10):
            person = mock.Person()
            person.name = "Name%d" % index
            person.save()

        result = mock.Person.find(limit = 5)

        self.assertEqual(len(result), 5)
        self.assertEqual(result[0].name, "Name0")

        result = mock.Person.find(skip = 2, limit = 5)

        self.assertEqual(len(result), 5)
        self.assertEqual(result[0].name, "Name2")

        result = mock.Person.find(skip = 3, limit = 20)

        self.assertEqual(len(result), 7)
        self.assertEqual(result[0].name, "Name3")

    def test_references(self):
        person = mock.Person()
        person.name = "Name"

        cat = mock.Cat()
        cat.name = "NameCat"
        cat.save()

        person.cats = [cat]
        person.save()

        person = mock.Person.get(identifier = 1)

        self.assertEqual(person.cats[0].name, "NameCat")

        person.cats = mock.Person.cats["type"]([cat])
        person.save()

        person = mock.Person.get(identifier = 1)

        self.assertEqual(person.cats.is_resolved(), False)
        self.assertEqual(person.car, None)
        self.assertEqual(person.cats[0].name, "NameCat")

        person = mock.Person.get(identifier = 1, map = True)

        self.assertEqual(isinstance(person, dict), True)
        self.assertEqual(isinstance(person["cats"], list), True)
        self.assertEqual(isinstance(person["cats"][0], int), True)
        self.assertEqual(len(person["cats"]), 1)

        person = mock.Person.get(identifier = 1, eager = ("cats",))

        self.assertEqual(person.cats.is_resolved(), True)

        person = mock.Person.get(
            identifier = 1,
            map = True,
            eager = ("cats",)
        )

        self.assertEqual(isinstance(person, dict), True)
        self.assertEqual(isinstance(person["cats"], list), True)
        self.assertEqual(isinstance(person["cats"][0], dict), True)

        person = mock.Person.get(identifier = 1)

        person.cats = []
        person.save()

        person = mock.Person.get(identifier = 1)

        self.assertEqual(len(person.cats), 0)

        person = mock.Person.get(map = True, eager = ("cats",))

        self.assertEqual(isinstance(person, dict), True)
        self.assertEqual(isinstance(person["cats"], list), True)
        self.assertEqual(len(person["cats"]), 0)

    def test_eager(self):
        person = mock.Person()
        person.name = "Name"
        person.save()

        car = mock.Car()
        car.name = "Car"
        car.save()

        garage = mock.Garage()
        garage.name = "Garage"
        garage.save()

        address = mock.Address()
        address.street = "Address"
        address.save()

        person = mock.Person.get(identifier = 1, eager_l = True)
        person.car = car
        person.save()

        car = mock.Car.get(identifier = 1, eager_l = True)
        car.garage = garage
        car.save()

        garage = mock.Garage.get(identifier = 1, eager_l = True)
        garage.address = address
        garage.save()

        person = mock.Person.get(identifier = 1, eager_l = True)

        self.assertEqual(isinstance(person.car, appier.Reference), True)
        self.assertEqual(person.car.is_resolved(), True)
        self.assertEqual(person.car.name, "Car")
        self.assertEqual(person.car.garage.is_resolved(), True)
        self.assertEqual(person.car.garage.name, "Garage")
        self.assertEqual(person.car.garage.address.is_resolved(), True)
        self.assertEqual(person.car.garage.address.street, "Address")

        person = mock.Person.get(identifier = 1, eager_l = False)

        self.assertEqual(isinstance(person.car, appier.Reference), True)
        self.assertEqual(person.car.is_resolved(), False)
        self.assertEqual(person.car.name, "Car")
        self.assertEqual(person.car.garage.is_resolved(), False)
        self.assertEqual(person.car.garage.name, "Garage")
        self.assertEqual(person.car.garage.address.is_resolved(), False)
        self.assertEqual(person.car.garage.address.street, "Address")

        person = mock.Person.get(identifier = 1)

        self.assertEqual(isinstance(person.car, appier.Reference), True)
        self.assertEqual(person.car.is_resolved(), False)
        self.assertEqual(person.car.name, "Car")
        self.assertEqual(person.car.garage.is_resolved(), False)
        self.assertEqual(person.car.garage.name, "Garage")
        self.assertEqual(person.car.garage.address.is_resolved(), False)
        self.assertEqual(person.car.garage.address.street, "Address")

        person = mock.Person.get(identifier = 1, map = True)

        self.assertEqual(person["car"]["name"], "Car")
        self.assertEqual(person["car"]["garage"]["name"], "Garage")
        self.assertEqual(person["car"]["garage"]["address"]["street"], "Address")

        person = mock.Person.get(identifier = 1, eager_l = True)

        person.car.name = "CarChanged"
        person.car.save()

        person = mock.Person.get(identifier = 1, eager_l = True)

        self.assertEqual(person.car.name, "CarChanged")

        father = mock.Person()
        father.name = "Father"
        father.save()

        car_father = mock.Car()
        car_father.name = "CarFather"
        car_father.save()

        father.car = car_father
        father.save()

        person.father = father
        person.save()

        person = mock.Person.get(identifier = 1, eager_l = True)

        self.assertEqual(isinstance(person.father, appier.Reference), True)
        self.assertEqual(person.father.is_resolved(), False)
        self.assertEqual(person.car.is_resolved(), True)

        person.father.resolve()

        self.assertEqual(person.car.is_resolved(), True)
        self.assertEqual(person.father.is_resolved(), True)
        self.assertEqual(person.father.car.is_resolved(), False)
        self.assertEqual(person.father.name, "Father")

        person.father.car.resolve()

        self.assertEqual(person.father.car.is_resolved(), True)
        self.assertEqual(person.father.car.name, "CarFather")

    def test_unresolvable(self):
        person = mock.Person()
        person.name = "Name"
        person.save()

        car = mock.Car()
        car.name = "Car"
        car.save()

        person = mock.Person.get(identifier = 1)
        person.car = car
        person.save()

        self.assertEqual(isinstance(person.car, appier.Reference), False)

        person = mock.Person.get(identifier = 1)

        self.assertEqual(isinstance(person.car, appier.Reference), True)
        self.assertEqual(person.car.is_resolvable(), True)
        self.assertEqual(person.car == None, False)
        self.assertEqual(person.car.name, "Car")

        car.delete()

        person = mock.Person.get(identifier = 1)

        self.assertEqual(isinstance(person.car, appier.Reference), True)
        self.assertEqual(person.car.is_resolvable(), False)
        self.assertEqual(person.car == None, False)

    def test_exists(self):
        person = mock.Person()
        person.name = "Name"

        self.assertEqual(person.exists(), False)

        person.save()

        self.assertEqual(person.exists(), True)

        person = mock.Person.get(name = "Name")

        self.assertEqual(person.exists(), True)

        person.delete()

        self.assertEqual(person.exists(), False)

    def test_wrap(self):
        person = mock.Person.wrap(dict(name = "Person"))
        self.assertEqual(person.name, "Person")

        person = mock.Person.wrap(dict(other = "Other"))
        self.assertEqual(person.other, "Other")

    def test_meta(self):
        self.assertEqual(appier.Model._to_meta(str), "string")
        self.assertEqual(appier.Model._to_meta(bool), "bool")
        self.assertEqual(appier.Model._to_meta(list), "list")
        self.assertEqual(appier.Model._to_meta(dict), "map")
        self.assertEqual(appier.Model._to_meta("text"), "text")
        self.assertEqual(appier.Model._to_meta("longtext"), "longtext")
        self.assertEqual(appier.Model._to_meta("longmap"), "longmap")
        self.assertEqual(appier.Model._to_meta(mock.Person.father["type"]), "reference")

    def test_meta_map(self):
        method = appier.model.METAS["map"]

        map = dict(hello = "world")
        result = method(map, {})

        self.assertEqual(type(result), str)
        self.assertEqual(result, "{\"hello\": \"world\"}")

        map = dict(mundo = "olá")

        self.assertEqual(type(result), str)
        self.assertEqual(method(map, {}), "{\"mundo\": \"olá\"}")

    def test_meta_longmap(self):
        method = appier.model.METAS["longmap"]

        map = dict(hello = "world")
        result = method(map, {})

        self.assertEqual(type(result), str)
        self.assertEqual(result, "{\"hello\": \"world\"}")

        map = dict(mundo = "olá")

        self.assertEqual(type(result), str)
        self.assertEqual(method(map, {}), "{\"mundo\": \"olá\"}")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import unittest

import appier

class RequestTest(unittest.TestCase):

    def test_mock(self):
        request = appier.Request(
            "GET",
            "/",
            address = "127.0.0.1",
            session_c = appier.FileSession
        )
        request.load_session()

        sid = request.session.sid

        self.assertEqual(request.session.__class__, appier.MockSession)
        self.assertEqual(request.session.sid, sid)
        self.assertEqual(request.session.address, "127.0.0.1")

        request.session["first"] = 1

        self.assertNotEqual(request.set_cookie, None)

        self.assertEqual(request.session["first"], 1)
        self.assertEqual(request.session.__class__, appier.FileSession)
        self.assertEqual(request.session.sid, sid)
        self.assertEqual(request.session.address, "127.0.0.1")

        appier.FileSession.close()

    def test_query_s(self):
        request = appier.Request(
            "GET",
            "/",
            query = "x-message=hello&message=world"
        )
        request.resolve_query_s()

        self.assertEqual(request.query, "x-message=hello&message=world")
        self.assertEqual(request.query_s, "message=world")
        self.assertEqual(request.location, "/")
        self.assertEqual(request.location_f, "/?message=world")

    def test_get_address(self):
        request = appier.Request("GET", "/", address = "127.0.0.1")

        self.assertEqual(request.get_address(), "127.0.0.1")

        request.in_headers["X-Forwarded-For"] = "1.1.1.1, 1.1.1.2, 1.1.1.3"
        self.assertEqual(request.get_address(), "1.1.1.1")

        request.in_headers["X-Client-Ip"] = "2.2.2.2"
        self.assertEqual(request.get_address(), "2.2.2.2")

        request.in_headers["X-Real-Ip"] = "3.3.3.3"
        self.assertEqual(request.get_address(), "3.3.3.3")

        result = request.get_address(resolve = False)
        self.assertEqual(result, "127.0.0.1")

    def test_get_header(self):
        request = appier.Request(
            "GET",
            "/",
            environ = dict(
                HTTP_CONTENT_TYPE = "text/plain",
                HTTP_X_REAL_IP = "1.1.1.1"
            )
        )
        request.load_headers()

        self.assertEqual(request.get_header("Content-Type"), "text/plain")
        self.assertEqual(request.get_header("content-type"), "text/plain")
        self.assertEqual(request.get_header("CONTENT-TYPE"), "text/plain")
        self.assertEqual(request.get_header("Content-TYPE"), "text/plain")
        self.assertEqual(request.get_header("Content_Type"), None)
        self.assertEqual(request.get_header("CONTENT_TYPE"), None)
        self.assertEqual(request.get_header("Content-Type", normalize = False), "text/plain")
        self.assertEqual(request.get_header("content-type", normalize = False), None)
        self.assertEqual(request.get_header("CONTENT-TYPE", normalize = False), None)

        self.assertEqual(request.get_header("X-Real-IP"), "1.1.1.1")
        self.assertEqual(request.get_header("X-Real-Ip"), "1.1.1.1")
        self.assertEqual(request.get_header("x-real-ip"), "1.1.1.1")
        self.assertEqual(request.get_header("X-REAL-IP"), "1.1.1.1")
        self.assertEqual(request.get_header("X-Real-IP"), "1.1.1.1")
        self.assertEqual(request.get_header("X_Real_Ip"), None)
        self.assertEqual(request.get_header("X_REAL_IP"), None)
        self.assertEqual(request.get_header("X-Real-Ip", normalize = False), "1.1.1.1")
        self.assertEqual(request.get_header("X-Real-IP", normalize = False), None)
        self.assertEqual(request.get_header("x-real-ip", normalize = False), None)
        self.assertEqual(request.get_header("X-REAL-IP", normalize = False), None)

    def test_data(self):
        request = appier.Request("GET", "/")
        request.set_data(appier.legacy.bytes("hello world", encoding = "utf-8"))

        self.assertEqual(type(request.get_data()), bytes)
        self.assertEqual(request.get_data(), appier.legacy.bytes("hello world", encoding = "utf-8"))
        self.assertEqual(type(request.get_encoded()), appier.legacy.UNICODE)
        self.assertEqual(request.get_encoded(), appier.legacy.u("hello world"))

        request = appier.Request("GET", "/")
        request.set_data(appier.legacy.bytes("你好世界", encoding = "utf-8"))

        self.assertEqual(type(request.get_data()), bytes)
        self.assertEqual(request.get_data(), appier.legacy.bytes("你好世界", encoding = "utf-8"))
        self.assertEqual(type(request.get_encoded()), appier.legacy.UNICODE)
        self.assertEqual(request.get_encoded(), appier.legacy.u("你好世界"))
        self.assertEqual(type(request.get_encoded(encoding = "ascii")), bytes)
        self.assertEqual(request.get_data(), appier.legacy.bytes("你好世界", encoding = "utf-8"))
        self.assertRaises(
            UnicodeDecodeError,
            lambda: request.get_encoded(encoding = "ascii", safe = False)
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import unittest

import appier

class CryptTest(unittest.TestCase):

    def test_rc4(self):
        rc4 = appier.RC4(b"hello key")
        result = rc4.encrypt(b"hello world")

        self.assertEqual(result, b"\xc54L\x00\xac\xb4\xf2\xcf\x8b5\xa7")

        rc4 = appier.RC4(b"hello key")
        data = rc4.decrypt(result)

        self.assertEqual(data, b"hello world")

        rc4 = appier.Cipher.new("rc4", b"hello key")
        result = rc4.encrypt(b"hello world")

        self.assertEqual(result, b"\xc54L\x00\xac\xb4\xf2\xcf\x8b5\xa7")

        rc4 = appier.Cipher.new("rc4", b"hello key")
        data = rc4.decrypt(result)

        self.assertEqual(data, b"hello world")

    def test_spritz(self):
        spritz = appier.Spritz(b"hello key")
        result = spritz.encrypt(b"hello world")

        self.assertEqual(result, b"\xbch\x0c\xb4X21\\\x07\xde\xe1")

        spritz = appier.Spritz(b"hello key")
        data = spritz.decrypt(result)

        self.assertEqual(data, b"hello world")

        spritz = appier.Cipher.new("spritz", b"hello key")
        result = spritz.encrypt(b"hello world")

        self.assertEqual(result, b"\xbch\x0c\xb4X21\\\x07\xde\xe1")

        spritz = appier.Cipher.new("spritz", b"hello key")
        data = spritz.decrypt(result)

        self.assertEqual(data, b"hello world")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import sys
import gzip

from . import http
from . import util
from . import legacy

class GeoResolver(object):

    DB_NAME = "GeoLite2-City.mmdb"
    """ The name of the file that contains the GeoIP
    information database (to be used in execution) """

    DOWNLOAD_URL = "http://geolite.maxmind.com/download/geoip/database/GeoLite2-City.mmdb.gz"
    """ The URL to the compressed version of the GeoIP
    city database used for resolution """

    VALID = ("continent", "country", "city", "location")
    """ The sequence of names that are considered to be valid
    under the simplified representation model """

    PREFIXES = ("", "~/", "/")
    """ The various prefixes that are going to be used in the
    search for the GeoIP database file, the order in which
    they are defined as they are search from the beginning """

    _db = None
    """ The reference to the internal database reference object
    that is going to be used in the GeoIP resolution """

    @classmethod
    def resolve(cls, address, simplified = True):
        db = cls._get_db()
        if not db: return None
        result = db.get(address)
        if simplified: result = cls._simplify(result)
        return result

    @classmethod
    def _simplify(cls, result, locale = "en", valid = VALID):
        if not result: return result
        for name, value in legacy.items(result):
            if not name in valid: del result[name]
            if not "names" in value: continue
            names = value["names"]
            value["name"] = names.get(locale, None)
            del value["names"]
        return result

    @classmethod
    def _get_db(cls):
        if cls._db: return cls._db
        maxminddb = util.import_pip("maxminddb")
        if not maxminddb: return None
        path = cls._try_all()
        if not path: return None
        cls._db = maxminddb.open_database(path)
        return cls._db

    @classmethod
    def _try_all(cls, prefixes = PREFIXES):
        for prefix in cls.PREFIXES:
            path = cls._try_db(path = prefix + cls.DB_NAME)
            if path: return path
        path = cls._try_db(path = cls.DB_NAME, download = True)
        if path: return path
        return None

    @classmethod
    def _try_db(cls, path = DB_NAME, download = False):
        path = os.path.expanduser(path)
        path = os.path.normpath(path)
        exists = os.path.exists(path)
        if exists: return path
        if not download: return None
        cls._download_db(path = path)
        exists = os.path.exists(path)
        if not exists: return None
        return path

    @classmethod
    def _download_db(cls, path = DB_NAME):
        contents = http.get(cls.DOWNLOAD_URL)
        cls._store_db(contents, path = path)

    @classmethod
    def _store_db(cls, contents, path = DB_NAME):
        path_gz = path + ".gz"
        file = open(path_gz, "wb")
        try: file.write(contents)
        finally: file.close()
        file = gzip.open(path_gz, "rb")
        try: contents = file.read()
        finally: file.close()
        file = open(path, "wb")
        try: file.write(contents)
        finally: file.close()
        os.remove(path_gz)
        return path

if __name__ == "__main__":
    prefix = "~/"
    if len(sys.argv) > 1: prefix = sys.argv[1]
    if not prefix.endswith("/"): prefix += "/"
    GeoResolver._try_db(
        path = prefix + GeoResolver.DB_NAME,
        download = True
    )
else:
    __path__ = []



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import json

from . import util
from . import config
from . import exceptions

try: import redis
except: redis = None

URL = "redis://localhost"
""" The default URL to be used for the connection when
no other URL is provided (used most of the times) """

connection = None
""" The global connection object that should persist
the connection relation with the database service """

class Redis(object):

    def __init__(self, url = None):
        self.url = url
        self._connection = None

    def get_connection(self, url = None):
        if self._connection: return self._connection
        url_c = config.conf("REDISTOGO_URL", None)
        url_c = config.conf("REDIS_URL", url_c)
        url = url or self.url or url_c or URL
        self._connection = _redis().from_url(url)
        return self._connection

def get_connection(url = URL):
    global connection
    if connection: return connection
    url = config.conf("REDISTOGO_URL", url)
    url = config.conf("REDIS_URL", url)
    connection = _redis().from_url(url)
    return connection

def dumps(*args):
    return json.dumps(*args)

def _redis(verify = True):
    if verify: util.verify(
        not redis == None,
        message = "redis library not available",
        exception = exceptions.OperationalError
    )
    return redis



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import unittest

import appier

class UtilTest(unittest.TestCase):

    def test_is_mobile(self):
        result = appier.is_mobile("Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19")
        self.assertEqual(result, True)

        result = appier.is_mobile("Mozilla/5.0 (iPhone; CPU iPhone OS 10_3 like Mac OS X) AppleWebKit/602.1.50 (KHTML, like Gecko) CriOS/56.0.2924.75 Mobile/14E5239e Safari/602.1")
        self.assertEqual(result, True)

        result = appier.is_mobile("Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/600.7.12 (KHTML, like Gecko) Version/8.0.7 Safari/600.7.12")
        self.assertEqual(result, False)

        result = appier.is_mobile("Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Build/KLP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30")
        self.assertEqual(result, False)

        result = appier.is_mobile("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.75 Safari/537.36")
        self.assertEqual(result, False)

        result = appier.is_mobile("Mozilla/5.0 (iPad; CPU OS 9_3_2 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13F69 Safari/601.1")
        self.assertEqual(result, False)

        result = appier.is_mobile("")
        self.assertEqual(result, False)

    def test_is_tablet(self):
        result = appier.is_tablet("Mozilla/5.0 (iPad; CPU OS 9_3_2 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13F69 Safari/601.1")
        self.assertEqual(result, True)

        result = appier.is_tablet("Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329")
        self.assertEqual(result, True)

        result = appier.is_tablet("Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19")
        self.assertEqual(result, True)

        result = appier.is_tablet("Mozilla/5.0 (iPhone; CPU iPhone OS 10_3 like Mac OS X) AppleWebKit/602.1.50 (KHTML, like Gecko) CriOS/56.0.2924.75 Mobile/14E5239e Safari/602.1")
        self.assertEqual(result, True)

        result = appier.is_tablet("Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Build/KLP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30")
        self.assertEqual(result, True)

        result = appier.is_tablet("Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/600.7.12 (KHTML, like Gecko) Version/8.0.7 Safari/600.7.12")
        self.assertEqual(result, False)

        result = appier.is_tablet("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.75 Safari/537.36")
        self.assertEqual(result, False)

        result = appier.is_tablet("")
        self.assertEqual(result, False)

    def test_is_browser(self):
        result = appier.is_browser("Mozilla/5.0 (iPad; CPU OS 9_3_2 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13F69 Safari/601.1")
        self.assertEqual(result, True)

        result = appier.is_browser("Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329")
        self.assertEqual(result, True)

        result = appier.is_browser("Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19")
        self.assertEqual(result, True)

        result = appier.is_browser("Mozilla/5.0 (iPhone; CPU iPhone OS 10_3 like Mac OS X) AppleWebKit/602.1.50 (KHTML, like Gecko) CriOS/56.0.2924.75 Mobile/14E5239e Safari/602.1")
        self.assertEqual(result, True)

        result = appier.is_browser("Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Build/KLP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30")
        self.assertEqual(result, True)

        result = appier.is_browser("Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/600.7.12 (KHTML, like Gecko) Version/8.0.7 Safari/600.7.12")
        self.assertEqual(result, True)

        result = appier.is_browser("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.75 Safari/537.36")
        self.assertEqual(result, True)

        result = appier.is_browser("Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.135 Safari/537.36 Edge/12.10136")
        self.assertEqual(result, True)

        result = appier.is_browser("DuckDuckBot/1.0; (+http://duckduckgo.com/duckduckbot.html)")
        self.assertEqual(result, False)

        result = appier.is_browser("netius/1.1.10")
        self.assertEqual(result, False)

        result = appier.is_browser("")
        self.assertEqual(result, False)

    def test_is_bot(self):
        result = appier.is_bot("Mozilla/5.0 (iPad; CPU OS 9_3_2 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13F69 Safari/601.1")
        self.assertEqual(result, False)

        result = appier.is_bot("Mozilla/5.0 (iPad; CPU OS 6_1_3 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Mobile/10B329")
        self.assertEqual(result, False)

        result = appier.is_bot("Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19")
        self.assertEqual(result, False)

        result = appier.is_bot("Mozilla/5.0 (iPhone; CPU iPhone OS 10_3 like Mac OS X) AppleWebKit/602.1.50 (KHTML, like Gecko) CriOS/56.0.2924.75 Mobile/14E5239e Safari/602.1")
        self.assertEqual(result, False)

        result = appier.is_bot("Mozilla/5.0 (Linux; U; Android 4.1.1; en-gb; Build/KLP) AppleWebKit/534.30 (KHTML, like Gecko) Version/4.0 Safari/534.30")
        self.assertEqual(result, False)

        result = appier.is_bot("Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_4) AppleWebKit/600.7.12 (KHTML, like Gecko) Version/8.0.7 Safari/600.7.12")
        self.assertEqual(result, False)

        result = appier.is_bot("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.75 Safari/537.36")
        self.assertEqual(result, False)

        result = appier.is_bot("Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.135 Safari/537.36 Edge/12.10136")
        self.assertEqual(result, False)

        result = appier.is_bot("DuckDuckBot/1.0; (+http://duckduckgo.com/duckduckbot.html)")
        self.assertEqual(result, True)

        result = appier.is_bot("netius/1.1.10")
        self.assertEqual(result, False)

        result = appier.is_bot("")
        self.assertEqual(result, False)

    def test_browser_info(self):
        result = appier.browser_info("Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.135 Safari/537.36 Edge/12.10136")
        self.assertEqual(result, dict(
            name = "Edge",
            version = "12.10136",
            version_f = 12.10136,
            version_i = 12,
            interactive = True,
            bot = False,
            os = "Windows"
        ))

        result = appier.browser_info("Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.75 Safari/537.36")
        self.assertEqual(result, dict(
            name = "Chrome",
            version = "62.0.3202.75",
            version_f = 62.0,
            version_i = 62,
            interactive = True,
            bot = False,
            os = "Windows"
        ))

        result = appier.browser_info("Mozilla/5.0 (iPad; CPU OS 9_3_2 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13F69 Safari/601.1")
        self.assertEqual(result, dict(
            name = "Safari",
            version = "601.1",
            version_f = 601.1,
            version_i = 601,
            interactive = True,
            bot = False,
            os = "Mac"
        ))

        result = appier.browser_info("Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:56.0) Gecko/20100101 Firefox/56.0")
        self.assertEqual(result, dict(
            name = "Firefox",
            version = "56.0",
            version_f = 56.0,
            version_i = 56,
            interactive = True,
            bot = False,
            os = "Windows"
        ))

        result = appier.browser_info("Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.0; Trident/4.0)")
        self.assertEqual(result, dict(
            name = "Explorer",
            version = "8.0",
            version_f = 8.0,
            version_i = 8,
            interactive = True,
            bot = False,
            os = "Windows"
        ))

        result = appier.browser_info("Mozilla/5.0 (compatible; Googlebot/2.1; +http://www.google.com/bot.html)")
        self.assertEqual(result, dict(
            name = "Googlebot",
            version = "2.1",
            version_f = 2.1,
            version_i = 2,
            interactive = False,
            bot = True
        ))

        result = appier.browser_info("Mozilla/5.0 (compatible; Bingbot/2.0; +http://www.bing.com/bingbot.htm)")
        self.assertEqual(result, dict(
            name = "Bingbot",
            version = "2.0",
            version_f = 2.0,
            version_i = 2,
            interactive = False,
            bot = True
        ))

        result = appier.browser_info("DuckDuckBot/1.0; (+http://duckduckgo.com/duckduckbot.html)")
        self.assertEqual(result, dict(
            name = "DuckDuckBot",
            version = "1.0",
            version_f = 1.0,
            version_i = 1,
            interactive = False,
            bot = True
        ))

        result = appier.browser_info("netius/1.1.10")
        self.assertEqual(result, dict(
            name = "netius",
            version = "1.1.10",
            version_f = 1.1,
            version_i = 1,
            interactive = False,
            bot = False
        ))

        result = appier.browser_info("APIs-Google (+https://developers.google.com/webmasters/APIs-Google.html)")
        self.assertEqual(result, None)

    def test_obfuscate(self):
        result = appier.obfuscate("hello world")
        self.assertEqual(result, "hel********")

        result = appier.obfuscate("hello world", display_l = 6)
        self.assertEqual(result, "hello *****")

        result = appier.obfuscate("hello world", display_l = 100)
        self.assertEqual(result, "hello world")

        result = appier.obfuscate("hello world", display_l = 6, token = "-")
        self.assertEqual(result, "hello -----")

        result = appier.obfuscate(appier.legacy.u("你好世界"), display_l = 3)
        self.assertEqual(result, appier.legacy.u("你好世*"))

    def test_email_parts(self):
        name, email = appier.email_parts("João Magalhães <joamag@hive.pt>")
        self.assertEqual(type(name), str)
        self.assertEqual(type(email), str)
        self.assertEqual(name, "João Magalhães")
        self.assertEqual(email, "joamag@hive.pt")

        name, email = appier.email_parts(appier.legacy.u("João Magalhães <joamag@hive.pt>"))
        self.assertEqual(type(name), appier.legacy.UNICODE)
        self.assertEqual(type(email), appier.legacy.UNICODE)
        self.assertEqual(name, appier.legacy.u("João Magalhães"))
        self.assertEqual(email, appier.legacy.u("joamag@hive.pt"))

        name, email = appier.email_parts(appier.legacy.u("你好世界 <hello@hive.pt>"))
        self.assertEqual(type(name), appier.legacy.UNICODE)
        self.assertEqual(type(email), appier.legacy.UNICODE)
        self.assertEqual(name, appier.legacy.u("你好世界"))
        self.assertEqual(email, appier.legacy.u("hello@hive.pt"))

    def test_email_mime(self):
        result = appier.email_mime("João Magalhães <joamag@hive.pt>")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "=?utf-8?q?Jo=C3=A3o_Magalh=C3=A3es?= <joamag@hive.pt>")

        result = appier.email_mime(appier.legacy.u("João Magalhães <joamag@hive.pt>"))
        self.assertEqual(type(result), appier.legacy.UNICODE)
        self.assertEqual(result, appier.legacy.u("=?utf-8?q?Jo=C3=A3o_Magalh=C3=A3es?= <joamag@hive.pt>"))

    def test_date_to_timestamp(self):
        result = appier.date_to_timestamp("29/06/1984")
        self.assertEqual(type(result), int)
        self.assertEqual(int(result), 457315200)

        result = appier.date_to_timestamp("29/06/0000")
        self.assertEqual(result, None)

        result = appier.date_to_timestamp("1984-06-29", format = "%Y-%m-%d")
        self.assertEqual(result, 457315200)

        result = appier.date_to_timestamp("1984-13-29", format = "%Y-%m-%d")
        self.assertEqual(result, None)

    def test_gather_errors(self):
        def raiser(): raise appier.OperationalError(message = "hello")
        struct = appier.lazy_dict(
            first = appier.lazy(lambda: raiser()),
            second = appier.lazy(lambda: 2),
        )

        errors = appier.gather_errors(struct)
        self.assertEqual(errors, dict(first = ["hello"]))

        struct.__getitem__("first", force = True)._value = 1

        errors = appier.gather_errors(struct)
        self.assertEqual(errors, dict(first = ["hello"]))

        struct.__getitem__("first", force = True)._value = 1

        errors = appier.gather_errors(struct, resolve = False)
        self.assertEqual(errors, dict())

    def test_camel_to_underscore(self):
        result = appier.camel_to_underscore("HelloWorld")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "hello_world")

        result = appier.camel_to_underscore("HELLOWorld")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "hello_world")

        result = appier.camel_to_underscore("HELLOWorldHELLOWorld")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "hello_world_hello_world")

    def test_camel_to_readable(self):
        result = appier.camel_to_readable("HelloWorld")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello World")

        result = appier.camel_to_readable("HelloWorld", lower = True)
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello world")

        result = appier.camel_to_readable("HelloWorld", lower = True, capitalize = True)
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello World")

        result = appier.camel_to_readable("HELLOWorld")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "HELLO World")

        result = appier.camel_to_readable("HELLOWorld", lower = True)
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello world")

        result = appier.camel_to_readable(
            "HELLOWorld",
            lower = True,
            capitalize = True
        )
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello World")

        result = appier.camel_to_readable("HELLOWorldHELLOWorld")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "HELLO World HELLO World")

        result = appier.camel_to_readable(
            "HELLOWorldHELLOWorld",
            lower = True
        )
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello world hello world")

        result = appier.camel_to_readable(
            "HELLOWorldHELLOWorld",
            lower = True,
            capitalize = True
        )
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello World Hello World")

    def test_underscore_to_readable(self):
        result = appier.underscore_to_readable("hello_world")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello world")

        result = appier.underscore_to_readable("hello_world", capitalize = True)
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello World")

        result = appier.underscore_to_readable("hello_world_hello_world")
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello world hello world")

        result = appier.underscore_to_readable("hello_world_hello_world", capitalize = True)
        self.assertEqual(type(result), str)
        self.assertEqual(result, "Hello World Hello World")

    def test_is_content_type(self):
        result = appier.is_content_type("text/plain", "text/plain")
        self.assertEqual(result, True)

        result = appier.is_content_type("text/plain", ("text/plain",))
        self.assertEqual(result, True)

        result = appier.is_content_type("text/plain", "text/html")
        self.assertEqual(result, False)

        result = appier.is_content_type("text/plain", ("text/html",))
        self.assertEqual(result, False)

        result = appier.is_content_type("text/plain", ("text/plain", "text/html"))
        self.assertEqual(result, True)

        result = appier.is_content_type("text/*", "text/plain")
        self.assertEqual(result, True)

        result = appier.is_content_type("text/*", "text/json")
        self.assertEqual(result, True)

    def test_parse_content_type(self):
        result = appier.parse_content_type("text/plain")
        self.assertEqual(type(result), tuple)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], ["text/plain"])
        self.assertEqual(result[1], dict())

        result = appier.parse_content_type("text/plain+json")
        self.assertEqual(type(result), tuple)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], ["text/plain", "text/json"])
        self.assertEqual(result[1], dict())

        result = appier.parse_content_type("text/plain+json; charset=utf-8")
        self.assertEqual(type(result), tuple)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], ["text/plain", "text/json"])
        self.assertEqual(result[1], dict(charset = "utf-8"))

        result = appier.parse_content_type("text/plain+json   ; charset=utf-8")
        self.assertEqual(type(result), tuple)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], ["text/plain", "text/json"])
        self.assertEqual(result[1], dict(charset = "utf-8"))

        result = appier.parse_content_type("text/plain+json; charset=utf-8; boundary=hello;")
        self.assertEqual(type(result), tuple)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], ["text/plain", "text/json"])
        self.assertEqual(result[1], dict(charset = "utf-8", boundary = "hello"))

        result = appier.parse_content_type("")
        self.assertEqual(type(result), tuple)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], [])
        self.assertEqual(result[1], dict())

        result = appier.parse_content_type("text")
        self.assertEqual(type(result), tuple)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], [])
        self.assertEqual(result[1], dict())

        result = appier.parse_content_type("text/plain+json; charset")
        self.assertEqual(type(result), tuple)
        self.assertEqual(len(result), 2)
        self.assertEqual(result[0], ["text/plain", "text/json"])
        self.assertEqual(result[1], dict())

    def test_check_login(self):
        request = appier.Request("GET", "/", session_c = appier.MemorySession)

        request.session["tokens"] = ["*"]
        result = appier.check_login(None, token = "admin", request = request)
        self.assertEqual(result, True)
        self.assertEqual(request.session["tokens"], {"*" : True})

        request.session["tokens"] = []
        result = appier.check_login(None, token = "admin", request = request)
        self.assertEqual(result, False)
        self.assertEqual(request.session["tokens"], {})

        request.session["tokens"] = ["admin"]
        result = appier.check_login(None, token = "admin", request = request)
        self.assertEqual(result, True)
        self.assertEqual(request.session["tokens"], {"admin" : True})

        request.session["tokens"] = ["admin.read"]
        result = appier.check_login(None, token = "admin", request = request)
        self.assertEqual(result, False)
        result = appier.check_login(None, token = "admin.read", request = request)
        self.assertEqual(result, True)
        self.assertEqual(request.session["tokens"], {
            "admin" : {
                "read" : True
            }
        })

        request.session["tokens"] = ["admin.*"]
        result = appier.check_login(None, token = "admin.read", request = request)
        self.assertEqual(result, True)
        self.assertEqual(request.session["tokens"], {
            "admin" : {
                "*" : True
            }
        })

        request.session["tokens"] = ["admin", "admin.write"]
        result = appier.check_login(None, token = "admin.read", request = request)
        self.assertEqual(result, False)
        self.assertEqual(request.session["tokens"], {
            "admin" : {
                "_" : True,
                "write" : True
            }
        })

        request.session["tokens"] = ["admin.write", "admin.*"]
        result = appier.check_login(None, token = "admin.read", request = request)
        self.assertEqual(result, True)
        self.assertEqual(request.session["tokens"], {
            "admin" : {
                "write" : True,
                "*" : True
            }
        })

        del request.session["tokens"]
        result = appier.check_login(None, token = "admin.read", request = request)
        self.assertEqual(result, False)
        self.assertEqual("tokens" in request.session, False)

    def test_check_tokens(self):
        result = appier.check_tokens(None, ("admin", "user"), tokens_m = {"*" : True})
        self.assertEqual(result, True)

        result = appier.check_tokens(None, ("admin", "user"), tokens_m = {})
        self.assertEqual(result, False)

        result = appier.check_tokens(None, ("admin", "user"), tokens_m = {"admin" : True})
        self.assertEqual(result, False)

    def test_check_token(self):
        result = appier.check_token(None, "admin", tokens_m = {"*" : True})
        self.assertEqual(result, True)

        result = appier.check_token(None, "admin", tokens_m = {})
        self.assertEqual(result, False)

        result = appier.check_token(None, "admin", tokens_m = {"admin" : True})
        self.assertEqual(result, True)

        result = appier.check_token(None, "admin.read", tokens_m = {
            "admin" : {
                "read" : True
            }
        })
        self.assertEqual(result, True)

        result = appier.check_token(None, "admin", tokens_m = {
            "admin" : {
                "read" : True
            }
        })
        self.assertEqual(result, False)

        result = appier.check_token(None, "admin.read", tokens_m = {
            "admin" : {
                "*" : True
            }
        })
        self.assertEqual(result, True)

        result = appier.check_token(None, None, tokens_m = {})
        self.assertEqual(result, True)

    def test_to_tokens_m(self):
        result = appier.to_tokens_m(["admin"])
        self.assertEqual(result, {"admin" : True})

        result = appier.to_tokens_m(["admin", "admin.read"])
        self.assertEqual(result, {
            "admin" : {
                "_" : True,
                "read" : True
            }
        })

        result = appier.to_tokens_m(["admin.read", "admin"])
        self.assertEqual(result, {
            "admin" : {
                "_" : True,
                "read" : True
            }
        })

        result = appier.to_tokens_m(["admin", "admin.*"])
        self.assertEqual(result, {
            "admin" : {
                "_" : True,
                "*" : True
            }
        })

    def test_dict_merge(self):
        first = dict(a = "hello", b = "world")
        second = dict(a = "hello_new", b = "world_new", c = "other")

        result = appier.dict_merge(first, second)
        self.assertEqual(id(result) in (id(first), (id(second))), False)
        self.assertEqual(result["a"], "hello_new")
        self.assertEqual(result["b"], "world_new")
        self.assertEqual(result["c"], "other")

        result = appier.dict_merge(first, second, override = False)
        self.assertEqual(id(result) in (id(first), (id(second))), False)
        self.assertEqual(result["a"], "hello")
        self.assertEqual(result["b"], "world")
        self.assertEqual(result["c"], "other")



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import csv
import uuid

from . import model
from . import legacy
from . import typesf
from . import exceptions

def serialize(obj):
    if isinstance(obj, model.Model): return obj.model
    if isinstance(obj, typesf.AbstractType): return obj.json_v()
    if type(obj) == type(None): return ""
    return legacy.UNICODE(obj)

def serialize_csv(items, encoding = "utf-8", delimiter = ";", strict = False):
    # verifies if the strict mode is active and there're no items defined
    # if that's the case an operational error is raised, otherwise an in
    # case the items are not provided the default (empty string) is returned
    if strict and not items: raise exceptions.OperationalError(
        message = "Empty items object provided, no keys available"
    )
    if not items: return str()

    # builds the encoder taking into account the provided encoding string
    # value, this encoder will be used to encode each of the partial values
    # that is going to be set in the target csv buffer
    encoder = build_encoder(encoding)

    # retrieves the first element and uses it to determine if the current
    # sequence to be serialized is map or sequence based
    first = items[0]
    is_map = type(first) == dict

    # retrieves the various keys from the first element of the provided sequence
    # of items then runs the eager operation (list loading) and sorts the provided
    # keys according to the default sorting order defined for the sequence, note
    # that in case the sequence is not map based the first element is ignored
    keys = first.keys() if is_map else first
    keys = legacy.eager(keys)
    if is_map: keys.sort()
    else: items = items[1:]

    # constructs the first row (names/keys row) using the gathered sequence of keys
    # and encoding them using the currently build encoder
    keys_row = [encoder(key) if type(key) == legacy.UNICODE else\
        key for key in keys]

    # creates the new string buffer and uses it as the basis for the construction of
    # the csv writer object, writing then the already build first row
    buffer = legacy.StringIO()
    writer = csv.writer(buffer, delimiter = delimiter)
    writer.writerow(keys_row)

    # iterates over the complete set of items to serialize each of it's attribute values
    # using the order defined in the keys sequence that has been defined, note that
    # this behavior is not applied to sequence based serialization (more linear)
    for item in items:
        # creates the list that is going to support the complete set of encoded
        # values (according to the defined encoding) and then start the base row
        # sequence with an empty value of the passed value (for map based values)
        row_e = []
        row = [] if is_map else item

        # iterates over the complete set of values in the map and serializes each
        # of them, adding then them to the base row sequence
        for key in keys if is_map else ():
            value = item[key]
            value = serialize(value)
            row.append(value)

        # iterates over the complete set of row values to "sanitize" them by encoding
        # them in case they are defined as unicode based values
        for value in row:
            is_unicode = isinstance(value, legacy.UNICODE)
            if is_unicode: value = encoder(value)
            row_e.append(value)

        # writes the encoded based row sequence to the writer in order to output the
        # values to the output based stream
        writer.writerow(row_e)

    # retrieves the buffer string value as the resulting value and returns it to the
    # caller method as the final result for the csv serialization
    result = buffer.getvalue()
    return result

def serialize_ics(items, encoding = "utf-8"):
    encoder = build_encoder(encoding)

    buffer = legacy.StringIO()
    buffer.write("BEGIN:VCALENDAR\r\n")
    buffer.write("METHOD:PUBLISH\r\n")
    buffer.write("X-WR-TIMEZONE:America/Los_Angeles\r\n")
    buffer.write("CALSCALE:GREGORIAN\r\n")
    buffer.write("VERSION:2.0\r\n")
    buffer.write("PRODID:-//PUC Calendar// v2.0//EN)\r\n")

    for item in items:
        start = item["start"]
        end = item["end"]
        description = item["description"]
        location = item["location"]
        timezone = item.get("timezone", "Etc/GMT")
        _uuid = item.get("uuid", None)
        _uuid = _uuid or str(uuid.uuid4())

        start = encoder(start)
        end = encoder(end)
        description = encoder(description)
        location = encoder(location)
        timezone = encoder(timezone)
        _uuid = encoder(_uuid)

        buffer.write("BEGIN:VEVENT\r\n")
        buffer.write("UID:%s\r\n" % _uuid)
        buffer.write("TZID:%s\r\n" % timezone)
        buffer.write("DTSTART:%s\r\n" % start)
        buffer.write("DTEND:%s\r\n" % end)
        buffer.write("DTSTAMP:%s\r\n" % start)
        buffer.write("SUMMARY:%s\r\n" % description)
        buffer.write("LOCATION:%s\r\n" % location)
        buffer.write("END:VEVENT\r\n")

    buffer.write("END:VCALENDAR\r\n")

    result = buffer.getvalue()
    return result

def build_encoder(encoding):
    if legacy.PYTHON_3: return lambda v: v
    else: return lambda v: v if v == None else v.encode(encoding)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import json

from . import util
from . import legacy
from . import typesf
from . import config
from . import common
from . import exceptions

try: import pymongo
except: pymongo = None

try: import bson.json_util
except: bson = None

URL = "mongodb://localhost"
""" The default URL to be used for the connection when
no other URL is provided (used most of the times) """

connection = None
""" The global connection object that should persist
the connection relation with the database service """

class Mongo(object):

    def __init__(self, url = None):
        self.url = url
        self._connection = None
        self._db = None

    def get_connection(self, url = None, connect = False):
        if self._connection: return self._connection
        url_c = config.conf("MONGOHQ_URL", None)
        url_c = config.conf("MONGOLAB_URI", url_c)
        url_c = config.conf("MONGO_URL", url_c)
        url = url or self.url or url_c or URL
        if is_new(): self._connection = _pymongo().MongoClient(url, connect = connect)
        else: self._connection = _pymongo().Connection(url)
        return self._connection

    def reset_connection(self):
        if not self._connection: return
        if is_new(): self._connection.close()
        else: self._connection.disconnect()
        self._connection = None

    def get_db(self, name):
        if self._db: return self._db
        connection = self.get_connection()
        self._db = connection[name]
        return self._db

class MongoEncoder(json.JSONEncoder):

    def default(self, obj, **kwargs):
        if not bson: return json.JSONEncoder.default(self, obj, **kwargs)
        if isinstance(obj, bson.objectid.ObjectId): return str(obj)
        if isinstance(obj, legacy.BYTES): return legacy.str(obj, encoding = "utf-8")
        else: return json.JSONEncoder.default(self, obj, **kwargs)

def get_connection(url = URL, connect = False):
    global connection
    if connection: return connection
    url = config.conf("MONGOHQ_URL", url)
    url = config.conf("MONGOLAB_URI", url)
    url = config.conf("MONGO_URL", url)
    if is_new(): connection = _pymongo().MongoClient(url, connect = connect)
    else: connection = _pymongo().Connection(url)
    return connection

def reset_connection():
    global connection
    if not connection: return
    if is_new(): connection.close()
    else: connection.disconnect()
    connection = None

def get_db(name = None):
    url = config.conf("MONGOHQ_URL", None)
    url = config.conf("MONGOLAB_URI", url)
    url = config.conf("MONGO_URL", url)
    result = legacy.urlparse(url or "")
    name = result.path.strip("/") if result.path else None
    name = name or config.conf("MONGO_DB", name)
    name = name or common.base().get_name() or "master"
    connection = get_connection()
    db = connection[name]
    return db

def drop_db(name = None):
    db = get_db(name = name)
    names = db.collection_names()
    for name in names:
        if name.startswith("system."): continue
        db.drop_collection(name)
    connection = get_connection()
    connection.drop_database(db.name)

def object_id(value):
    return bson.ObjectId(value)

def dumps(*args):
    return json.dumps(default = serialize, *args)

def serialize(obj):
    if isinstance(obj, common.model().Model): return obj.model
    if isinstance(obj, typesf.AbstractType): return obj.json_v()
    return bson.json_util.default(obj)

def directions(all = False):
    return (
        _pymongo().ASCENDING,
        _pymongo().DESCENDING,
        _pymongo().HASHED
    ) if all else (
        _pymongo().ASCENDING,
        _pymongo().DESCENDING
    )

def is_mongo(obj):
    if bson and isinstance(obj, bson.ObjectId): return True
    if bson and isinstance(obj, bson.DBRef): return True
    return False

def is_new():
    return int(_pymongo().version[0]) >= 3 if pymongo else False

def _store_find_and_modify(store, *args, **kwargs):
    if is_new(): store.find_one_and_update(*args, **kwargs)
    else: store.find_and_modify(*args, **kwargs)

def _store_insert(store, *args, **kwargs):
    if is_new(): store.insert_one(*args, **kwargs)
    else: store.insert(*args, **kwargs)

def _store_update(store, *args, **kwargs):
    if is_new(): store.update_one(*args, **kwargs)
    else: store.update(*args, **kwargs)

def _store_remove(store, *args, **kwargs):
    if is_new(): store.delete_many(*args, **kwargs)
    else: store.remove(*args, **kwargs)

def _store_ensure_index(store, *args, **kwargs):
    kwargs["background"] = kwargs.get("background", True)
    if is_new(): store.create_index(*args, **kwargs)
    else: store.ensure_index(*args, **kwargs)

def _store_ensure_index_many(store, *args, **kwargs):
    directions_l = kwargs.pop("directions", None)
    if directions_l == "all": directions_l = directions(all = True)
    elif directions_l == None: directions_l = directions()
    for direction in directions_l:
        _args = list(args)
        _args[0] = [(_args[0], direction)]
        _store_ensure_index(store, *_args, **kwargs)

def _pymongo(verify = True):
    if verify: util.verify(
        not pymongo == None,
        message = "pymongo library not available",
        exception = exceptions.OperationalError
    )
    return pymongo



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import unittest

import appier

class QueuingTest(unittest.TestCase):

    def test_memory(self):
        queue = appier.MemoryQueue()
        queue.clear()
        queue.push("hello")
        result = queue.pop()

        self.assertEqual(queue.length(), 0)
        self.assertEqual(result, "hello")

        identifier = queue.push("hello")
        queue.pop()

        self.assertEqual(identifier, None)
        self.assertEqual(result, "hello")

        identifier = queue.push("hello", identify = True)

        self.assertNotEqual(identifier, None)

        priority, _identifier, result = queue.pop(full = True)

        self.assertEqual(priority, None)
        self.assertEqual(_identifier, identifier)
        self.assertEqual(result, "hello")

        identifier_1 = queue.push("hello 1", priority = 3, identify = True)
        identifier_2 = queue.push("hello 2", priority = 1, identify = True)
        identifier_3 = queue.push("hello 3", priority = 5, identify = True)

        self.assertEqual(queue.length(), 3)

        _priority, _identifier_3, _result_3 = queue.pop(full = True)
        _priority, _identifier_1, _result_1 = queue.pop(full = True)
        _priority, _identifier_2, _result_2 = queue.pop(full = True)

        self.assertEqual(_result_1, "hello 1")
        self.assertEqual(_result_2, "hello 2")
        self.assertEqual(_result_3, "hello 3")
        self.assertEqual(_identifier_1, identifier_1)
        self.assertEqual(_identifier_2, identifier_2)
        self.assertEqual(_identifier_3, identifier_3)

    def test_multiprocess(self):
        queue = appier.MultiprocessQueue()
        queue.clear()
        queue.push("hello")
        result = queue.pop()

        self.assertEqual(queue.length(), 0)
        self.assertEqual(result, "hello")

        identifier = queue.push("hello")
        queue.pop()

        self.assertEqual(identifier, None)
        self.assertEqual(result, "hello")

        identifier = queue.push("hello", identify = True)

        self.assertNotEqual(identifier, None)

        priority, _identifier, result = queue.pop(full = True)

        self.assertEqual(priority, None)
        self.assertEqual(_identifier, identifier)
        self.assertEqual(result, "hello")

        identifier_1 = queue.push("hello 1", priority = 3, identify = True)
        identifier_2 = queue.push("hello 2", priority = 1, identify = True)
        identifier_3 = queue.push("hello 3", priority = 5, identify = True)

        self.assertEqual(queue.length(), 3)

        _priority, _identifier_3, _result_3 = queue.pop(full = True)
        _priority, _identifier_1, _result_1 = queue.pop(full = True)
        _priority, _identifier_2, _result_2 = queue.pop(full = True)

        self.assertEqual(_result_1, "hello 1")
        self.assertEqual(_result_2, "hello 2")
        self.assertEqual(_result_3, "hello 3")
        self.assertEqual(_identifier_1, identifier_1)
        self.assertEqual(_identifier_2, identifier_2)
        self.assertEqual(_identifier_3, identifier_3)

    def test_amqp(self):
        try: queue = appier.AMQPQueue()
        except:
            if not hasattr(self, "skipTest"): return
            self.skipTest("No AMQP server present")

        queue.clear()
        queue.push("hello")
        result = queue.pop()

        self.assertEqual(result, "hello")

        identifier = queue.push("hello")
        queue.pop()

        self.assertEqual(identifier, None)
        self.assertEqual(result, "hello")

        identifier = queue.push("hello", identify = True)

        self.assertNotEqual(identifier, None)

        priority, _identifier, result = queue.pop(full = True)

        self.assertEqual(priority, None)
        self.assertEqual(_identifier, identifier)
        self.assertEqual(result, "hello")

        identifier_1 = queue.push("hello 1", priority = 3, identify = True)
        identifier_2 = queue.push("hello 2", priority = 1, identify = True)
        identifier_3 = queue.push("hello 3", priority = 5, identify = True)

        _priority, _identifier_3, _result_3 = queue.pop(full = True)
        _priority, _identifier_1, _result_1 = queue.pop(full = True)
        _priority, _identifier_2, _result_2 = queue.pop(full = True)

        self.assertEqual(_result_1, "hello 1")
        self.assertEqual(_result_2, "hello 2")
        self.assertEqual(_result_3, "hello 3")
        self.assertEqual(_identifier_1, identifier_1)
        self.assertEqual(_identifier_2, identifier_2)
        self.assertEqual(_identifier_3, identifier_3)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Appier Framework
# Copyright (c) 2008-2017 Hive Solutions Lda.
#
# This file is part of Hive Appier Framework.
#
# Hive Appier Framework is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Appier Framework is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Appier Framework. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2017 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import util
from . import common

class Component(object):
    """
    Top level component class to be used as an abstract definition
    of a loadable/unloadable component of the system.

    This provides a strict (yet simple) set of rules to how such
    component should behave and the life-cycle stages for it.
    """

    def __init__(self, name = "component", owner = None, *args, **kwargs):
        object.__init__(self)
        self.id = util.gen_token()
        self.name = name
        self.owner = owner or common.base().APP
        self.loaded = False

    def load(self, *args, **kwargs):
        self._load(*args, **kwargs)
        return self

    def unload(self, *args, **kwargs):
        self._unload(*args, **kwargs)
        return self

    @property
    def is_loaded(self):
        return self.loaded

    def _load(self, *args, **kwargs):
        self.loaded = True

    def _unload(self, *args, **kwargs):
        self.owner = None
        self.loaded = False
